<?php

namespace Teknisa\SM\Controller;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Row;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\Sm00049000AgrupamentoContratosFaturamentoService;
use Doctrine\ORM\EntityManager;
use Teknisa\SM\Util\Utilities;

class Sm00049000AgrupamentoContratosFaturamentoController extends SMCrud {
    
    protected $dataSourceName = "fct_agrupamento";
    protected $agrupamentoService;
    protected $connection;
    protected $newCode;
    
    public function  __construct($crudService, $environment, Sm00049000AgrupamentoContratosFaturamentoService $agrupamentoService, NewCode $newCode, EntityManager $entityManager){
        parent::__construct($crudService, $environment);
        $this->agrupamentoService = $agrupamentoService;
        $this->newCode            = $newCode;
        $this->connection         = $entityManager->getConnection();
    }
    
    public function find(Request\Filter $request, Response $response) {
        try {
            parent::find($request, $response);
            $erro = $response->getError();
            if ($erro) {
                throw new \Exception($erro->getMessage(), $erro->getErrorCode());
            }
            
            $rows = $response->getDataSets()[0]->getRows();
            foreach ($rows as &$row) {
                $row["NRTIPOORCAMENTO"] = Utilities::getArrayDeString($row["NRTIPOORCAMENTO"], true);
            }
            unset($row);
            $response->setDataSets(array(new DataSet("fct_agrupamento", $rows)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Manda validar se os contratos selecionados para o agrupamento possuem 
     * as mesma parametrizações.
    **/
    public function validarContratosAgrupamento(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrseqcontratos = $row["NRSEQCONTRATO"];
            $nragrupamento  = $row["NRAGRUPAMENTO"];
            $nrorg   = $this->environment->getNrorg();
            $dataSet = $this->agrupamentoService->validarContratosAgrupamento($nrseqcontratos, $nrorg, $nragrupamento);
            
            if($dataSet !== null)
                $response->addDataSet(new DataSet("fct_relacagrupamentocont_widget", $dataSet));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Manda regerar o faturamento de n contratos aprovados ligados a um agrupamento
    **/
    public function salvaContratosAgrupamento(Request\Row $request, Response $response){
        try {
            
            $row = $request->getRow();
            
            $nrseqcontratos = $row["NRSEQCONTRATO"];
            $nragrupamento = $row["NRAGRUPAMENTO"];
            
            $nrorg    = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            
            $this->agrupamentoService->salvaContratosAgrupamento($nrseqcontratos, $nragrupamento, $nrorg, $operador);
               
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida se os escopos de faturamento possuem informações similares
    **/
    public function validarEscoposAgrupamento(Request\Row $request, Response $response) {
        try {
            $row   = $request->getRow();
            $nrorg = $this->environment->getNrorg();
            
            $escopos = $row["NRTIPOORCAMENTO"];

            $possuiescsimilares = $this->agrupamentoService->validarEscoposAgrupamento($escopos, $nrorg);
            $response->addDataSet(new DataSet("fct_agrupamento_widget", array("IDESCSIMILARES" => $possuiescsimilares)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Salva um agrupamento
    **/
    public function salvarAgrupamento(Request\Row $request, Response $response) {
        try {
            $row   = $request->getRow();
            $nrorg      = $this->environment->getNrorg();
            $cdoperador = $this->environment->getCdOperador();
            
            $escopos = $this->agrupamentoService->getInformacoesEscopos($row["NRTIPOORCAMENTO"], $nrorg);
            $row["CDTIPOOPERFATURA"] = $escopos["CDTIPOOPERFATURA"];
            $row["CDSERIENF"]        = $escopos["CDSERIENF"];
            $row["CDEMPRESA"]        = $escopos["CDEMPRESA"];
            $row["CDINSCESTA"]       = $escopos["CDINSCESTA"];
            $row["CDINSCMUNI"]       = $escopos["CDINSCMUNI"];
            $row["CDFILIALFATURA"]   = $escopos["CDFILIALFATURA"];
            
            if (!$row["__is_new"]) {
                parent::save($request, $response);
            } else {
                $request->setParameter("dbname", "fct_agrupamento");
                
                $nragrupamento        = $this->newCode->getCodeByTableName("FCT_AGRUPAMENTO", $nrorg, 1, 10, true);
                $row["NRAGRUPAMENTO"] = $nragrupamento;
                
                parent::save($request, $response);
                $columnsToReturn = $this->crudService->getColumnsToReturn();
                $columnsToReturn[0]["NRAGRUPAMENTO"] = $nragrupamento;
                $columnsToReturn[0]["NRORG"]         = $nrorg;
                $response->addDataSet(new DataSet("columnsToReturn", $columnsToReturn));
                
                $this->agrupamentoService->salvarEscoposAgrupamento($row["NRTIPOORCAMENTO"], $nragrupamento, $cdoperador, $nrorg);
            }
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}