/*****************************************************************************
* Sistemas Integrados TECNISA                                                *
*                                                                            *
* Compras e Suprimentos                                                      *
*                                                                            *
******************************************************************************
*                                                                            *
* Nome: MENSAL_DATA_ENTR_MES                                                 *
*                                                                            *
* Descrio:                                                                 *
* Calcula datas de entrega para um determinado mes baseando-se na            *
* Periodicidade de Entrega Mensal e nos Feriados de Compras.                 *
*                                                                            *
*                                                                            *
* Criao:                                                                   *
* 06/01/2000 - Yoris L. de Souza                                               *
*                                                                            *
* Manutenes:                                                               *
******************************************************************************

******************************************************************************
* PARAMETROS:                                                                *
*                                                                            *
* P_CDFILIAL     - Cdigo da Filial                                          *
* P_CDPERIENTR   - Cdigo da Periodicidade de Entrega                        *
* P_DTINICIAL    - Data do Inicio do Mes                                     *
* P_IDDIANUTIL   - Considera dia nao util                                    *
******************************************************************************/

/* @D:\SQL\CALECOMP\8030_CMPSTP_MES_DATA_ENTR_MES.SQL  */

IF EXISTS(SELECT * FROM sysobjects O, sysusers U
          WHERE 
            o.name = 'MENSAL_DATA_ENTR_MES'  and 
            type = 'P'                  and
            u.uid = o.uid               and 
            u.name = USER)
  DROP PROCEDURE MENSAL_DATA_ENTR_MES
  GO

CREATE PROCEDURE MENSAL_DATA_ENTR_MES
   @P_CDFILIAL     VARCHAR(4),
   @P_CDPERIENTR   VARCHAR(2),
   @P_DTINICIAL    DATETIME,
   @P_IDDIANUTIL   VARCHAR(1),
   @P_DTPROCESSA   DATETIME
AS
   DECLARE @V_CONTAUXI   NUMERIC(10);
   DECLARE @V_DIENTREGA  VARCHAR(2);
   DECLARE @V_DTDIAENTR  DATETIME;
   DECLARE @V_IDDIASEM   INT;
   DECLARE @V_DTENTRAUX  DATETIME;
   DECLARE @V_PRIMVEZ    NUMERIC(10);
   DECLARE @V_DIPRIMVEZ  INT;
   DECLARE @V_JAEXISTE   NUMERIC(10);
   DECLARE @V_DTINIAUX   DATETIME;
   DECLARE @V_DTENTREGA_PROX DATETIME;
   DECLARE @V_DTENTREGA_ANT DATETIME;


/*-------- CURSOR QUE CONTEM TODOS OS DIAS DE ENTREGA DE DETERMINADA PERIODICIDADE ------*/
    DECLARE C_DIAS_ENTR CURSOR FOR
      SELECT DIENTREGA
      FROM DIASENTR
      WHERE
        CDPERIENTR = @P_CDPERIENTR
      ORDER BY DIENTREGA;

BEGIN
    SET @V_DTINIAUX = @P_DTINICIAL;
    SET @V_DTENTREGA_ANT = NULL;

    SET @V_DTINIAUX = DATEADD(MM, 1, @V_DTINIAUX) - DAY(DATEADD(MM, 1, @V_DTINIAUX));/*LAST_DAY*/

    SET @V_DTENTRAUX = @P_DTINICIAL;

    OPEN C_DIAS_ENTR;
    FETCH NEXT FROM C_DIAS_ENTR INTO @V_DIENTREGA;
    WHILE (@@FETCH_STATUS = 0)
    BEGIN

     SET @V_DTDIAENTR = (@P_DTINICIAL - 1) + CONVERT(INT,@V_DIENTREGA);

     IF (@V_DTDIAENTR > @V_DTINIAUX)
     BEGIN
        SET @V_DTDIAENTR = @V_DTINIAUX;
     END;

     IF (@V_DTDIAENTR >= @P_DTPROCESSA)
     BEGIN
   /*- CALCULA UMA DATA DE ENTREGA ANTERIOR A UMA DATA NAO UTIL -*/
        IF (@P_IDDIANUTIL = 'S')
        BEGIN
           SET @V_CONTAUXI = 1;
           WHILE (@V_CONTAUXI > 0)
           BEGIN
             SET @V_CONTAUXI = 0;
             SELECT @V_CONTAUXI = COUNT(*)
             FROM FERICOMP
             WHERE
             CDFILIAL   = @P_CDFILIAL AND
             DTFERICOMP = @V_DTDIAENTR;
             IF (@V_CONTAUXI > 0)
             BEGIN
                SET @V_DTDIAENTR = @V_DTDIAENTR - 1;
             END;
           END;
        END;
   /* - Verifica se j existe a data de entrega que foi calculada por um dia no util -*/
        SET @V_JAEXISTE = 0;
        SELECT @V_JAEXISTE = COUNT(*)
        FROM CALECOMP
        WHERE
          CDFILIAL   = @P_CDFILIAL AND
          CDPERIENTR = @P_CDPERIENTR AND
          DTENTREGA  = @V_DTDIAENTR;
   /*- Se j existir a data pula -*/
        IF (@V_JAEXISTE = 0)
        BEGIN

     /*- VERIFICA SE JA EXISTE ALGUMA DATA DE ENTREGA CALCULADA PARA  -*/
     /*- INCLUIR A DATA ANTERIOR A PROXIMA ENTREGA  -*/

           SET @V_PRIMVEZ = 0;
           SELECT @V_PRIMVEZ = COUNT(*)
           FROM CALECOMP
           WHERE
            CDFILIAL   = @P_CDFILIAL AND
            CDPERIENTR = @P_CDPERIENTR AND
            DTENTREGA  = @V_DTENTRAUX;

     /*- INCLUI A DATA ANTERIOR A PROXIMA ENTREGA  -
           IF (@V_PRIMVEZ > 0)
           BEGIN
              UPDATE CALECOMP
              SET DTANTPROXENT = @V_DTDIAENTR - 1
                WHERE CDFILIAL   = @P_CDFILIAL AND
                      CDPERIENTR = @P_CDPERIENTR AND
                      DTENTREGA  = @V_DTENTRAUX;
           END
           ELSE
           BEGIN
              SET @V_DIPRIMVEZ = @V_DIENTREGA;
           END;

     - INCLUI A DATA DE ENTREGA CALCULADA NO CALENDARIO DE COMPRAS -*/
           INSERT INTO CALECOMP
             (CDFILIAL,
              CDPERIENTR,
              DTENTREGA,
              DTANTPROXENT)
             VALUES
             (@P_CDFILIAL,
              @P_CDPERIENTR,
              @V_DTDIAENTR,
              @V_DTENTRAUX);

           SET @V_DTENTRAUX = @V_DTDIAENTR;
        END;

        UPDATE CALECOMP
        SET DTANTPROXENT = @V_DTDIAENTR - 1
        WHERE
          CDFILIAL   = @P_CDFILIAL AND
          CDPERIENTR = @P_CDPERIENTR AND
          DTENTREGA  = @V_DTENTREGA_ANT;

        SET @V_DTENTREGA_ANT = @V_DTDIAENTR;
     END;
     FETCH NEXT FROM C_DIAS_ENTR INTO @V_DIENTREGA;
    END;
    CLOSE C_DIAS_ENTR;
    DEALLOCATE C_DIAS_ENTR;

    -- Calcula a data anterior a prxima entrega da ltima data de entrega calculada.

    SET @V_DIENTREGA = '00';
    SELECT
      @V_DIENTREGA = MIN(DIENTREGA)
    FROM
      DIASENTR
    WHERE
      CDPERIENTR = @P_CDPERIENTR;

    SET @V_DTENTREGA_PROX = @P_DTINICIAL;

    SET @V_DTENTREGA_PROX = DATEADD(MM, 1, @V_DTENTREGA_PROX) - DAY(DATEADD(MM, 1, @V_DTENTREGA_PROX));/*LAST_DAY*/

    SET @V_DTENTREGA_PROX = @V_DTENTREGA_PROX + CONVERT(INT, @V_DIENTREGA);

   /*- CALCULA UMA DATA DE ENTREGA ANTERIOR A UMA DATA NAO UTIL -*/
    IF (@P_IDDIANUTIL = 'S')
    BEGIN
       SET @V_CONTAUXI = 1;
       WHILE (@V_CONTAUXI > 0)
       BEGIN
         SET @V_CONTAUXI = 0;
         SELECT @V_CONTAUXI = COUNT(*)
         FROM FERICOMP
         WHERE
         CDFILIAL   = @P_CDFILIAL AND
         DTFERICOMP = @V_DTENTREGA_PROX;
         IF (@V_CONTAUXI > 0)
         BEGIN
            SET @V_DTENTREGA_PROX = @V_DTENTREGA_PROX - 1;
         END;
       END;
    END;

    UPDATE CALECOMP
    SET DTANTPROXENT = @V_DTENTREGA_PROX - 1
    WHERE
      CDFILIAL   = @P_CDFILIAL AND
      CDPERIENTR = @P_CDPERIENTR AND
      DTENTREGA  = @V_DTENTREGA_ANT;
/*
    -- PEGA A PRIMEIRA DATA ANTERIOR A PROXIMA ENTREGA
    SET @V_DTDIAENTR = @V_DTINIAUX + @V_DIPRIMVEZ;

    SET @V_DTINIAUX = @V_DTINIAUX + 1
    EXEC ULTIMO_DIA_MES @P_DTDATA = @V_DTINIAUX OUTPUT;
    IF (@V_DTDIAENTR > @V_DTINIAUX)
    BEGIN
       SET @V_DTDIAENTR = @V_DTINIAUX;
    END;

    IF (@V_DTDIAENTR >= @P_DTPROCESSA)
    BEGIN

   -- CALCULA UMA DATA DE ENTREGA ANTERIOR A UMA DATA NAO UTIL
       IF (@P_IDDIANUTIL = 'S')
       BEGIN
           SET @V_CONTAUXI = 1;
           WHILE (@V_CONTAUXI > 0)
           BEGIN
             SELECT @V_CONTAUXI = COUNT(*)
             FROM FERICOMP
             WHERE
             CDFILIAL   = @P_CDFILIAL AND
             DTFERICOMP = @V_DTDIAENTR;
             IF (@V_CONTAUXI > 0)
             BEGIN
                SET @V_DTDIAENTR = @V_DTDIAENTR - 1;
             END;
           END;
        END;
        SET @V_DTDIAENTR = @V_DTDIAENTR - 1;

   -- VERIFICA SE JA EXISTE A ULTIMA DATA DE ENTREGA QUE RECEBERA
   -- A DATA ANTERIOR A PROXIMA ENTREGA

       SELECT @V_PRIMVEZ = COUNT(*)
       FROM CALECOMP
       WHERE
         CDFILIAL   = @P_CDFILIAL AND
         CDPERIENTR = @P_CDPERIENTR AND
         DTENTREGA  = @V_DTENTRAUX;

       IF (@V_PRIMVEZ > 0)
       BEGIN
   -- INCLUI A DATA ANTERIOR A PROXIMA ENTREGA PARA A A ULTIMA DATA DE ENTREGA DO MES
         UPDATE CALECOMP
           SET DTANTPROXENT = @V_DTDIAENTR
           WHERE CDFILIAL   = @P_CDFILIAL AND
                 CDPERIENTR = @P_CDPERIENTR AND
                 DTENTREGA  = @V_DTENTRAUX;
       END;
    END; */
END;
GO